--
-- AFTER THIS SCRIPT HAS BEEN GENERATED YOU MUST RUN ./mgmt/commandline/short.pl TO ESTABLISH SHORT NAMES
-- FOR ALL ASSOCIATED HANDLES.  FAILURE TO DO THIS WILL PREVENT CERTAIN ASPECTS OF ETHEREAL REALMS FROM
-- WORKING PROPERLY.   PLEASE NOTE THAT SOME HANDLES MAY BE ERASED.
--

--
-- MAIL ENHANCEMENTS
--

-- DROP OLD READ TAG
ALTER TABLE PGSCHEMA.Mail DROP MailRead;

-- CREATE NEW ZONES
ALTER TABLE PGSCHEMA.Mail ADD MailFromDel  VARCHAR(3);
ALTER TABLE PGSCHEMA.Mail ADD MailRcptRead VARCHAR(3);
ALTER TABLE PGSCHEMA.Mail ADD MailRcptDel  VARCHAR(3);

-- SET APPROPRIATE DEFAULTS
UPDATE PGSCHEMA.Mail SET MailFromDel='no';
UPDATE PGSCHEMA.Mail SET MailRcptRead='no';
UPDATE PGSCHEMA.Mail SET MailRcptDel='no';

-- SET DATA CONTRAINTS
ALTER TABLE PGSCHEMA.Mail ALTER COLUMN MailFromDel  SET NOT NULL;
ALTER TABLE PGSCHEMA.Mail ALTER COLUMN MailRcptRead SET NOT NULL;
ALTER TABLE PGSCHEMA.Mail ALTER COLUMN MailRcptDel  SET NOT NULL;

ALTER TABLE PGSCHEMA.Mail ALTER COLUMN MailFromDel  SET DEFAULT 'no';
ALTER TABLE PGSCHEMA.Mail ALTER COLUMN MailRcptRead SET DEFAULT 'no';
ALTER TABLE PGSCHEMA.Mail ALTER COLUMN MailRcptDel  SET DEFAULT 'no';

ALTER TABLE PGSCHEMA.Mail ADD CONSTRAINT enumMailFromDel CHECK (MailFromDel IN ('yes','no'));
ALTER TABLE PGSCHEMA.Mail ADD CONSTRAINT enumMailRcptRead CHECK (MailRcptRead IN ('yes','no'));
ALTER TABLE PGSCHEMA.Mail ADD CONSTRAINT enumMailRcptDel CHECK (MailRcptDel IN ('yes','no'));

-- INDEXES
CREATE INDEX idxMail_Rcpt ON PGSCHEMA.Mail
(
       MailRcptLogin,
       MailRcptDel
);

CREATE INDEX idxMail_From ON PGSCHEMA.Mail
(
       MailFromLogin,
       MailFromDel
);

--
-- MAIL TRACKING ENHANCEMENTS
--
ALTER TABLE PGSCHEMA.Mail ALTER COLUMN PuppeteerLogin RENAME MailRcptLogin;
ALTER TABLE PGSCHEMA.Mail ADD COLUMN MailFromLogin VARCHAR(10);


--
-- HANDLE CHANGES
--
ALTER TABLE PGSCHEMA.Puppet ADD COLUMN PuppetShort VARCHAR(30);

CREATE INDEX idxPuppet_ShortName ON PGSCHEMA.Puppet
(
  PuppetShort
) WHERE PuppetPool='system';


--
--
-- NAME        : Liason
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : This table keeps track of the relations between two individuals
--               first by tracking who has contacted who first and secondly by
--               allowing a user to change the settings from neutral to friend
--               and/or foe.

CREATE TABLE PGSCHEMA.Liason (
  LiasonOwner          VARCHAR(10)   NOT NULL,
  LiasonTarget         VARCHAR(10)   NOT NULL,
  LiasonSource         VARCHAR(6)    NOT NULL DEFAULT 'owner',
  LiasonRelation       VARCHAR(7)    NOT NULL DEFAULT 'neutral',
  CONSTRAINT PkLiason                PRIMARY KEY (LiasonOwner, LiasonTarget),
  CONSTRAINT enumLiasonSource        CHECK (LiasonSource   IN ('owner','target','nil')),
  CONSTRAINT enumLiasonRelation      CHECK (LiasonRelation IN ('neutral','friend','foe')),
  CONSTRAINT FkLiasonOwner           FOREIGN KEY (LiasonOwner)
    REFERENCES PGSCHEMA.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT FkLiasonTarget          FOREIGN KEY (LiasonTarget)
    REFERENCES PGSCHEMA.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


ALTER TABLE PGSCHEMA.Liason                OWNER TO PGOWNER;

--
--
-- NAME        : Vanity
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : This table is related heavily to the social network aspect of the
--               site as well.   Simply put, this table stores all the necessary
--               information to display a users pictures

CREATE SEQUENCE PGSCHEMA.seqVanity   MAXVALUE 2147483647;

CREATE TABLE PGSCHEMA.Vanity (
  VanityID             INT           NOT NULL DEFAULT NEXTVAL('PGSCHEMA.seqVanity'),
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  VanityPath           VARCHAR(200)  NOT NULL,
  VanityThumb          VARCHAR(200)  NOT NULL,
  VanityWidth          SMALLINT      NOT NULL,
  VanityHeight         SMALLINT      NOT NULL,
  VanityDefault        VARCHAR(3)    NOT NULL DEFAULT 'no',
  VanityPublic         VARCHAR(3)    NOT NULL DEFAULT 'yes',
  VanityVote           VARCHAR(3)    NOT NULL DEFAULT 'no',
  CONSTRAINT pkVanity                PRIMARY KEY (VanityID),
  CONSTRAINT enumVanityDefault       CHECK (VanityDefault IN ('yes','no')),
  CONSTRAINT enumVanityPublic        CHECK (VanityPublic  IN ('yes','no')),
  CONSTRAINT enumVanityVote          CHECK (VanityVote    IN ('yes','no')),
  CONSTRAINT FkVanity                FOREIGN KEY (PuppeteerLogin)
    REFERENCES PGSCHEMA.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
);

ALTER TABLE PGSCHEMA.Vanity                OWNER TO PGOWNER;
ALTER TABLE PGSCHEMA.seqVanity             OWNER TO PGOWNER;


--
--
-- NAME        : Vanity
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : This table is related heavily to the social network aspect of the
--               site as well.   Simply put, this table stores all the necessary
--               information to display a users pictures

CREATE TABLE PGSCHEMA.MailVanity (
  MailIDNumber         INT           NOT NULL,
  VanityID             INT           NOT NULL,
  CONSTRAINT pkMailVanity            PRIMARY KEY (MailIDNumber, VanityID),
  CONSTRAINT fkMailVanity_Mail       FOREIGN KEY (MailIDNumber)
    REFERENCES PGSCHEMA.Mail (MailIDNumber)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkMailVanity_Vanity     FOREIGN KEY (VanityID)
    REFERENCES PGSCHEMA.Vanity (VanityID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Vanity Votes
-- REFERENCES  : Puppeteer
--               Vanity
--
-- DESCRIPTION : Vanity votes will very simply keep track of votes by user for
--               a specific picture.

CREATE TABLE PGSCHEMA.VanityVotes (
  VanityID             INT           NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  VoteLevel            SMALLINT      NOT NULL,
  CONSTRAINT fkVanityVote_Login      FOREIGN KEY (PuppeteerLogin)
    REFERENCES PGSCHEMA.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkVanityVote_VanityID   FOREIGN KEY (VanityID)
    REFERENCES PGSCHEMA.Vanity (VanityID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
);


-- Additional tag types for use
ALTER TABLE PGSCHEMA.Tag DROP CONSTRAINT enumTagType;
ALTER TABLE PGSCHEMA.Tag ADD  CONSTRAINT enumTagType CHECK (TagType IN ('admin','board','bio','configuration','contrib','faq','gallery','gummy','login','mail','poll','portal','realm','system','template','user','vanity','vcodes'));

ALTER TABLE PGSCHEMA.Gummy DROP CONSTRAINT enumGummyClass;
ALTER TABLE PGSCHEMA.Gummy ADD CONSTRAINT enumGummyClass          CHECK (GummyClass IN ('admin','default','canon','contributor','general','inject','mod','super'));


--
--
CREATE OR REPLACE FUNCTION PGSCHEMA.trigWork () RETURNS TRIGGER AS '
  BEGIN

    --
    -- REMOVE
    IF TG_OP = ''DELETE'' THEN

       -- Update Gallery
       EXECUTE ''UPDATE PGSCHEMA.Gallery''
            || '' SET GalleryHighlight=NULL,''
            || ''     GalleryThumbnail=NULL''
            || '' WHERE GalleryID = '' || QUOTE_LITERAL(OLD.GalleryID);


      -- Return row unmodified
      RETURN OLD;


    --
    -- UPDATE
    ELSIF TG_OP = ''UPDATE''  THEN

       -- This should be for image works only
       IF NEW.WorkImage IS NOT NULL THEN

         -- This is a child realm
         IF NEW.WorkName <> OLD.WorkName OR NEW.WorkImageThumbnail <> OLD.WorkImageThumbnail THEN

             -- Make sure nothing is highlighted
             IF OLD.WorkLayer IS NOT NULL AND NEW.WorkLayer IS NOT NULL THEN

                -- Update Layers
                EXECUTE ''UPDATE PGSCHEMA.Layer''
                     || '' SET LayerHighlight='' || QUOTE_LITERAL(NEW.WorkName) || '',''
                     || ''     LayerThumbnail='' || QUOTE_LITERAL(NEW.WorkImageThumbnail)
                     || '' WHERE LayerHighlight = '' || QUOTE_LITERAL(OLD.WorkName)
                     || ''   AND LayerThumbnail  = '' || QUOTE_LITERAL(OLD.WorkImageThumbnail)
                     || ''   AND LayerID = '' || QUOTE_LITERAL(NEW.WorkLayer)
                     || ''   AND GalleryID = '' || QUOTE_LITERAL(NEW.GalleryID);

             END IF;

             -- Update Gallery
             EXECUTE ''UPDATE PGSCHEMA.Gallery''
                  || '' SET GalleryHighlight = '' || QUOTE_LITERAL(NEW.WorkName) || '',''
                  || ''     GalleryThumbnail = '' || QUOTE_LITERAL(NEW.WorkImageThumbnail)
                  || '' WHERE GalleryHighlight = '' || QUOTE_LITERAL(OLD.WorkName)
                  || ''   AND GalleryThumbnail  = '' || QUOTE_LITERAL(OLD.WorkImageThumbnail)
                  || ''   AND GalleryID = '' || QUOTE_LITERAL(NEW.GalleryID);
   
         END IF;
       END IF;

       -- Just pass it along
       RETURN NEW;

    END IF;


  END;
 ' LANGUAGE 'plpgsql';

--
--
-- NAME        : Social
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Additional component for the social networking system
--               that deals with the additional information required for
--               that system.

CREATE TABLE PGSCHEMA.Social (
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  LocateCity           VARCHAR(45),
  LocateRegion         VARCHAR(45),
  LocateCountry        VARCHAR(45),
  LocateLat            REAL,
  LocateLong           REAL,
  PhysicalGender       VARCHAR(10),
  PhysicalBirthDate    TIMESTAMP,
  PhysicalHeight       VARCHAR(15),
  PhysicalBodyType     VARCHAR(15),
  PhysicalEyeColour    VARCHAR(15),
  PhysicalHairColour   VARCHAR(15),
  PhysicalEhtnicity    VARCHAR(15),
  IntroCatchPhrase     TEXT,
  IntroDescription     TEXT,
  IntroFirstDate       TEXT,
  SocialLooking        VARCHAR(15),
  SocialSeeking        VARCHAR(10),
  SocialMarital        VARCHAR(10),
  SocialChildrenHave   VARCHAR(10),
  SocialChildrenWant   VARCHAR(10),
  SocialInterests      TEXT,
  SocialReligion       VARCHAR(30),
  SocialProfession     VARCHAR(30),
  ViceSmoke            VARCHAR(10),
  ViceDrink            VARCHAR(10),
  CONSTRAINT pkSocial                PRIMARY KEY (PuppeteerLogin),
  CONSTRAINT valLocateLat            CHECK (LocateLat  <= 90  AND LocateLat  >= -90),
  CONSTRAINT valLocateLong           CHECK (LocateLong <= 180 AND LocateLong >= -180),
  CONSTRAINT fkSocial                FOREIGN KEY (PuppeteerLogin)
    REFERENCES PGSCHEMA.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;



--
--
-- FUNCTION    : getAge
--
-- DESCRIPTION : This will return a properly formatted Date stamp for use
--               in the outside world.

CREATE OR REPLACE FUNCTION PGSCHEMA.getAge(TIMESTAMP) RETURNS INT AS'
    SELECT EXTRACT(YEAR FROM AGE($1))::INT
 ' LANGUAGE SQL IMMUTABLE STRICT;

--
--
-- FUNCTION    : getBrithDay
--
-- DESCRIPTION : Fetches the day from the Birth Date.

CREATE OR REPLACE FUNCTION PGSCHEMA.getBirthDay(TIMESTAMP) RETURNS INT AS'
  SELECT EXTRACT(DAY FROM $1)::INT
 ' LANGUAGE SQL IMMUTABLE STRICT;

--
--
-- FUNCTION    : getBrithMonth
--
-- DESCRIPTION : Fetches the day from the Birth Month.

CREATE OR REPLACE FUNCTION PGSCHEMA.getBirthMonth(TIMESTAMP) RETURNS INT AS'
  SELECT EXTRACT(MONTH FROM $1)::INT
 ' LANGUAGE SQL IMMUTABLE STRICT;

--
--
-- FUNCTION    : getBrithYear
--
-- DESCRIPTION : Fetches the day from the Birth Year.

CREATE OR REPLACE FUNCTION PGSCHEMA.getBirthYear(TIMESTAMP) RETURNS INT AS'
  SELECT EXTRACT(YEAR FROM $1)::INT
 ' LANGUAGE SQL IMMUTABLE STRICT;




ALTER TABLE PGSCHEMA.Social ALTER COLUMN LocateCity     SET NOT NULL;
ALTER TABLE PGSCHEMA.Social ALTER COLUMN LocateRegion   SET NOT NULL;
ALTER TABLE PGSCHEMA.social ALTER COLUMN LocateCountry  SET NOT NULL;
ALTER TABLE PGSCHEMA.Social ALTER COLUMN LocateLat      SET NOT NULL;
ALTER TABLE PGSCHEMA.Social ALTER COLUMN LocateLong     SET NOT NULL;

ALTER TABLE PGSCHEMA.Social ADD SocialAgeMin SMALLINT;
ALTER TABLE PGSCHEMA.Social ADD SocialAgeMax SMALLINT;

ALTER TABLE PGSCHEMA.Social ALTER COLUMN SocialAgeMin SET DEFAULT 18;
ALTER TABLE PGSCHEMA.Social ALTER COLUMN SocialAgeMax SET DEFAULT 99;

ALTER TABLE PGSCHEMA.Social ADD CONSTRAINT SocialAgeMin CHECK (SocialAgeMin > 17 AND SocialAgeMin < 100);
ALTER TABLE PGSCHEMA.Social ADD CONSTRAINT SocialAgeMax CHECK (SocialAgeMax > 17 AND SocialAgeMax < 100);


--
--
-- NAME        : Network
--
-- DESCRIPTION : This combines every table necessary to create a single point of entry
--               for searching and viewing the social network.  

CREATE VIEW PGSCHEMA.Network AS
   SELECT
     p.PuppeteerLogin       AS PuppeteerLogin,
     p.PuppeteerTimestamp   AS PuppeteerTimestamp,
     h.PuppetName           AS PuppetName,
     h.PuppetShort          AS PuppetShort,
     s.LocateCity           AS LocateCity,
     s.LocateRegion         AS LocateRegion,
     s.LocateCountry        AS LocateCountry,
     s.LocateLat            AS LocateLat,
     s.LocateLong           AS LocateLong,
     s.PhysicalGender       AS PhysicalGender,
     s.PhysicalBirthDate    AS PhysicalBirthDate,
     s.PhysicalHeight       AS PhysicalHeight,
     s.PhysicalBodyType     AS PhysicalBodyType,
     s.PhysicalEyeColour    AS PhysicalEyeColour,
     s.PhysicalHairColour   AS PhysicalHairColour,
     s.PhysicalEthnicity    AS PhysicalEthnicity,
     s.IntroCatchPhrase     AS IntroCatchPhrase,
     s.IntroDescription     AS IntroDescription,
     s.IntroFirstDate       AS IntroFirstDate,
     s.SocialLooking        AS SocialLooking,
     s.SocialSeeking        AS SocialSeeking,
     s.SocialAgeMin         AS SocialAgeMin,
     s.SocialAgeMax         AS SocialAgeMax,
     s.SocialMarital        AS SocialMarital,
     s.SocialChildrenHave   AS SocialChildrenHave,
     s.SocialChildrenWant   AS SocialChildrenWant,
     s.SocialInterests      AS SocialInterests,
     s.SocialReligion       AS SocialReligion,
     s.SocialProfession     AS SocialProfession,
     s.ViceSmoke            AS ViceSmoke,
     s.ViceDrink            AS ViceDrink,
     PGSCHEMA.getBirthDay(s.PhysicalBirthDate)   AS PhysicalBirthDay,
     PGSCHEMA.getBirthMonth(s.PhysicalBirthDate) AS PhysicalBirthMonth,
     PGSCHEMA.getBirthYear(s.PhysicalBirthDate)  AS PhysicalBirthYear,
     PGSCHEMA.getAge(s.PhysicalBirthDate)        AS PhysicalAge
  FROM PGSCHEMA.Social s, PGSCHEMA.Puppeteer p, PGSCHEMA.Puppet h
  WHERE (s.PuppeteerLogin = h.PuppeteerLogin
    AND  h.PuppetPool     = 'system'
    AND  h.PuppetDefault  = 'yes')
    AND s.PuppeteerLogin = P.PuppeteerLogin;



--
--
-- NAME        : Astrology
-- REFERENCES  : NIL
--
-- DESCRIPTION : European astrological signs.

CREATE TABLE PGSCHEMA.Astrology (
  AstroSign          VARCHAR(15)   NOT NULL,
  AstroStart         DATE          NOT NULL,
  AstroEnd           DATE          NOT NULL,
  CONSTRAINT pkAstrology           PRIMARY KEY (AstroSign, AstroStart)
);

-- STANDARD INDEX
CREATE INDEX idxAstrology ON PGSCHEMA.Astrology
(
  AstroStart,
  AstroEnd
);

INSERT INTO PGSCHEMA.Tag VALUES ('TagInterestsList', 'vanity','<b>MNAME</b>');
INSERT INTO PGSCHEMA.Tag VALUES ('TagInterestsNone', 'vanity','<b>None</b>');
INSERT INTO PGSCHEMA.Tag VALUES ('TxtFirst', 'vanity','&lt;&lt;');
INSERT INTO PGSCHEMA.Tag VALUES ('TxtPrev', 'vanity','&lt;');
INSERT INTO PGSCHEMA.Tag VALUES ('TxtNext', 'vanity','&gt;');
INSERT INTO PGSCHEMA.Tag VALUES ('TxtLast', 'vanity','&gt;&gt;');
INSERT INTO PGSCHEMA.Tag VALUES ('SetDisplayMax', 'vanity','250');
INSERT INTO PGSCHEMA.Tag VALUES ('TmplSocialSearchList', 'vanity','');